#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
    /// Improved MACD BB Indicator
	/// 
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>
	/// 
    [Description("Enter the description of your new custom indicator here")]
    public class ImpMACDBB : Indicator
    {
        #region Variables
        // Wizard generated variables
        private int fast = 12; // Default setting for Fast
        private int slow = 26; // Default setting for Slow
        private int smooth = 9; // Default setting for Smooth
        private int bandPeriod = 20; // Default setting for BandPeriod
		private double numDevs = 1; // Default setting for NumDevs
		
		private Color risingMACD = Color.Green;
		private Color fallingMACD = Color.Red;
        // User defined variables (add any user defined variables below)
		private MACD macd;
		private StdDev std;
		private SMA avg, avgDiff;
		private DataSeries diff;
		private int barsBetweenBands = 0, totalBars = 0;
        #endregion

        protected override void Initialize()
        {
			Add(new Line(Color.FromKnownColor(KnownColor.Black), 0, "Zero"));
			
            Add(new Plot(Color.FromKnownColor(KnownColor.Yellow), PlotStyle.Dot, "MACD"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Black), PlotStyle.Dot, "MACDFrame"));
			
            Add(new Plot(Color.FromKnownColor(KnownColor.Cyan), PlotStyle.Line, "Average"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Cyan), PlotStyle.Line, "Upper"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Cyan), PlotStyle.Line, "Lower"));
			
			Plots[0].Pen.DashStyle = DashStyle.Dot;
			Plots[1].Pen.DashStyle = DashStyle.Dot;
			Plots[2].Pen.DashStyle = DashStyle.Dot; 
			
			Plots[0].Pen.Width = 2;
			Plots[1].Pen.Width = 3;
            
			Overlay				= false;
			DrawOnPricePanel = false;
        }
		
		protected override void OnStartUp()
		{
			diff = new DataSeries(this);
						
			macd = MACD(fast, slow, smooth);
			avg = SMA(macd, bandPeriod);
			std = StdDev(diff, bandPeriod);
			avgDiff = SMA(diff, bandPeriod);
		}

        protected override void OnBarUpdate()
        {
			double hDiff = macd[0] - avg[0];
			double lDiff = avg[0] - macd[0];			
            diff[0] = Math.Max(hDiff, lDiff);			
			
			MACDPlot[0] = macd[0];
			MACDFrame[0] = macd[0];
			Average[0] = avg[0];
			Upper[0] = avg[0] + avgDiff[0] + numDevs * std[0];
			Lower[0] = avg[0] - avgDiff[0] - numDevs * std[0];
			
			if (Rising(MACDPlot))
				PlotColors[0][0] = risingMACD;
			else if (Falling(MACDPlot))
				PlotColors[0][0] = fallingMACD;			
        }				

        #region Properties
		
		#region Plots
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MACDPlot
        {
            get { return Values[0]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MACDFrame
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Average
        {
            get { return Values[2]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Upper
        {
            get { return Values[3]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Lower
        {
            get { return Values[4]; }
        }
		
		#endregion
		
		#region Parameters

        [Description("")]
        [GridCategory("Parameters")]
        public int Fast
        {
            get { return fast; }
            set { fast = Math.Max(1, value); }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int Slow
        {
            get { return slow; }
            set { slow = Math.Max(1, value); }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int Smooth
        {
            get { return smooth; }
            set { smooth = Math.Max(1, value); }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int BandPeriod
        {
            get { return bandPeriod; }
            set { bandPeriod = Math.Max(1, value); }
        }
		
		[Description("")]
        [GridCategory("Parameters")]
        public double NumDevs
        {
            get { return numDevs; }
            set { numDevs = value; }
        }
		
		#endregion
		
		#region Colors
		
		[XmlIgnore()]
		[Description("")]
		[GridCategory("Plot Colors")]		
		public Color RisingMACD
		{
			get { return risingMACD; }
			set { risingMACD = value; }
		}
		
		[Browsable(false)]
		public string RisingMACDS
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(risingMACD); }
			set { risingMACD = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[XmlIgnore()]
		[Description("")]
		[GridCategory("Plot Colors")]		
		public Color FallingMACD
		{
			get { return fallingMACD; }
			set { fallingMACD = value; }
		}
		
		[Browsable(false)]
		public string FallingMACDS
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(fallingMACD); }
			set { fallingMACD = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		#endregion
		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private ImpMACDBB[] cacheImpMACDBB = null;

        private static ImpMACDBB checkImpMACDBB = new ImpMACDBB();

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public ImpMACDBB ImpMACDBB(int bandPeriod, Color fallingMACD, int fast, double numDevs, Color risingMACD, int slow, int smooth)
        {
            return ImpMACDBB(Input, bandPeriod, fallingMACD, fast, numDevs, risingMACD, slow, smooth);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public ImpMACDBB ImpMACDBB(Data.IDataSeries input, int bandPeriod, Color fallingMACD, int fast, double numDevs, Color risingMACD, int slow, int smooth)
        {
            if (cacheImpMACDBB != null)
                for (int idx = 0; idx < cacheImpMACDBB.Length; idx++)
                    if (cacheImpMACDBB[idx].BandPeriod == bandPeriod && cacheImpMACDBB[idx].FallingMACD == fallingMACD && cacheImpMACDBB[idx].Fast == fast && Math.Abs(cacheImpMACDBB[idx].NumDevs - numDevs) <= double.Epsilon && cacheImpMACDBB[idx].RisingMACD == risingMACD && cacheImpMACDBB[idx].Slow == slow && cacheImpMACDBB[idx].Smooth == smooth && cacheImpMACDBB[idx].EqualsInput(input))
                        return cacheImpMACDBB[idx];

            lock (checkImpMACDBB)
            {
                checkImpMACDBB.BandPeriod = bandPeriod;
                bandPeriod = checkImpMACDBB.BandPeriod;
                checkImpMACDBB.FallingMACD = fallingMACD;
                fallingMACD = checkImpMACDBB.FallingMACD;
                checkImpMACDBB.Fast = fast;
                fast = checkImpMACDBB.Fast;
                checkImpMACDBB.NumDevs = numDevs;
                numDevs = checkImpMACDBB.NumDevs;
                checkImpMACDBB.RisingMACD = risingMACD;
                risingMACD = checkImpMACDBB.RisingMACD;
                checkImpMACDBB.Slow = slow;
                slow = checkImpMACDBB.Slow;
                checkImpMACDBB.Smooth = smooth;
                smooth = checkImpMACDBB.Smooth;

                if (cacheImpMACDBB != null)
                    for (int idx = 0; idx < cacheImpMACDBB.Length; idx++)
                        if (cacheImpMACDBB[idx].BandPeriod == bandPeriod && cacheImpMACDBB[idx].FallingMACD == fallingMACD && cacheImpMACDBB[idx].Fast == fast && Math.Abs(cacheImpMACDBB[idx].NumDevs - numDevs) <= double.Epsilon && cacheImpMACDBB[idx].RisingMACD == risingMACD && cacheImpMACDBB[idx].Slow == slow && cacheImpMACDBB[idx].Smooth == smooth && cacheImpMACDBB[idx].EqualsInput(input))
                            return cacheImpMACDBB[idx];

                ImpMACDBB indicator = new ImpMACDBB();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.BandPeriod = bandPeriod;
                indicator.FallingMACD = fallingMACD;
                indicator.Fast = fast;
                indicator.NumDevs = numDevs;
                indicator.RisingMACD = risingMACD;
                indicator.Slow = slow;
                indicator.Smooth = smooth;
                Indicators.Add(indicator);
                indicator.SetUp();

                ImpMACDBB[] tmp = new ImpMACDBB[cacheImpMACDBB == null ? 1 : cacheImpMACDBB.Length + 1];
                if (cacheImpMACDBB != null)
                    cacheImpMACDBB.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheImpMACDBB = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ImpMACDBB ImpMACDBB(int bandPeriod, Color fallingMACD, int fast, double numDevs, Color risingMACD, int slow, int smooth)
        {
            return _indicator.ImpMACDBB(Input, bandPeriod, fallingMACD, fast, numDevs, risingMACD, slow, smooth);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.ImpMACDBB ImpMACDBB(Data.IDataSeries input, int bandPeriod, Color fallingMACD, int fast, double numDevs, Color risingMACD, int slow, int smooth)
        {
            return _indicator.ImpMACDBB(input, bandPeriod, fallingMACD, fast, numDevs, risingMACD, slow, smooth);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ImpMACDBB ImpMACDBB(int bandPeriod, Color fallingMACD, int fast, double numDevs, Color risingMACD, int slow, int smooth)
        {
            return _indicator.ImpMACDBB(Input, bandPeriod, fallingMACD, fast, numDevs, risingMACD, slow, smooth);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.ImpMACDBB ImpMACDBB(Data.IDataSeries input, int bandPeriod, Color fallingMACD, int fast, double numDevs, Color risingMACD, int slow, int smooth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.ImpMACDBB(input, bandPeriod, fallingMACD, fast, numDevs, risingMACD, slow, smooth);
        }
    }
}
#endregion
